unit SjrdDateTimeUtils;

interface

{$I SJRDUnits.inc}

uses
{$IFDEF SJRD_WINDOWS}
  Controls,
{$ENDIF}
{$IFDEF SJRD_LINUX}
  QControls,
{$ENDIF}
  SysUtils;

type

  TUsualDate = record
  case integer of
    0 : (Year : Word; Month : Byte; Day : Byte);
    1 : (Date : LongWord);
  end;

function GetUsualDate(Year : Word; Month, Day : Byte) : TUsualDate;
// Renvoie le TUsualDate contenant la date passe en paramtres

function UsualDateToSysDate(UsualDate : TUsualDate) : TDate;
// Transforme un TUsualDate en un TDate

function SysDateToUsualDate(SysDate : TDate) : TUsualDate;
// Transforme un TDate en un TUsualDate

function DateAddition(Date : TUsualDate; Days : integer) : TUsualDate;
// Revoie la date Date avec Days jours en plus
// Days peut tre ngatif, auquel cas les jours seront retranchs

function CompareDates(Date1, Date2 : TUsualDate) : integer;
// Compare Date1 et Date2
// Renvoie -1 si Date1 < Date2 ; 0 si Date1 = Date2 ; 1 si Date1 > Date2

function NumberOfDays(Month : Byte; Year : Word = 0) : integer;
// Renvoie le nombre de jours qu'il y a dans le moi Month en l'an Year
// Year est utilis uniquement si Month = 2 ; si Year = 0, alors on prend en
// compte l'anne courante

function GetMonthName(Month : integer; Maj : boolean = True) : string;
// Revoie le nom du mois Month (1 pour Janvier). Maj dtermine s'il y a une majuscule

implementation

function GetUsualDate(Year : Word; Month, Day : Byte) : TUsualDate;
begin
  Result.Year := Year;
  Result.Month := Month;
  Result.Day := Day;
end;

function UsualDateToSysDate(UsualDate : TUsualDate) : TDate;
begin
  Result := EncodeDate(UsualDate.Year, UsualDate.Month, UsualDate.Day);
end;

function SysDateToUsualDate(SysDate : TDate) : TUsualDate;
var AMonth, ADay : Word;
begin
  DecodeDate(SysDate, Result.Year, AMonth, ADay);
  Result.Month := AMonth;
  Result.Day := ADay;
end;

function DateAddition(Date : TUsualDate; Days : integer) : TUsualDate;
begin
  // Initialise Result  Date
  Result.Date := Date.Date;

  // Ajoute d'abord la nombre de jours  Result.Day
  inc(Result.Day, Days);

  // Tant que le nombre de jours est trop important :
  while Result.Day > NumberOfDays(Result.Month, Result.Year) do
  begin
    // On retire le nombre de jours du mois courant
    dec(Result.Day, NumberOfDays(Result.Month, Result.Year));
    // On incrmente ensuite le mois courant
    inc(Result.Month);
    // Si le mois courant est suprieur  12 alors :
    if Result.Month > 12 then
    begin
      // Le mois devient Janvier
      Result.Month := 1;
      // On augmente l'anne
      inc(Result.Year);
    end;
  end;

  // Tant que le nombre de jours est infrieur  1 :
  while Result.Day < 1 do
  begin
    // On retire un mois
    dec(Result.Month);
    // Si le mois est infrieur  1 alors :
    if Result.Month < 1 then
    begin
      // Le mois devient Dcembre
      Result.Month := 12;
      // On dcrmente l'anne
      dec(Result.Year);
    end;
    // On ajoute le nombre de jours du mois courant
    inc(Result.Day, NumberOfDays(Result.Month, Result.Year));
  end;
end;

function CompareDates(Date1, Date2 : TUsualDate) : integer;
begin
  if Date1.Year  < Date2.Year  then Result := -1 else
  if Date1.Year  > Date2.Year  then Result :=  1 else
  if Date1.Month < Date2.Month then Result := -1 else
  if Date1.Month > Date2.Month then Result :=  1 else
  if Date1.Day   < Date2.Day   then Result := -1 else
  if Date1.Day   > Date2.Day   then Result :=  1 else
  Result := 0;
end;

function NumberOfDays(Month : Byte; Year : Word = 0) : integer;
const
  NumDays : array[1..12] of Byte =
    (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
begin
  if Month = 2 then
  begin
    // Si le mois est Fvrier, il faut vrifier si l'anne est bissextile ou pas
    // Si elle l'est (si l'anne est div par 4 ET si l'anne n'est pas div par
    // 100 ou galement div par 400), il y a 29 jours sinon il n'y en a que 28
    if Year = 0 then Year := SysDateToUsualDate(Date).Year;
    if (Year mod 4 = 0) and ((Year mod 100 <> 0) or (Year mod 400 = 0)) then
      Result := 29 else Result := 28;
  end else Result := NumDays[Month];
end;

function GetMonthName(Month : integer; Maj : boolean = True) : string;
begin
  case Month of
    1 : Result := 'janvier';
    2 : Result := 'fvrier';
    3 : Result := 'mars';
    4 : Result := 'avril';
    5 : Result := 'mai';
    6 : Result := 'juin';
    7 : Result := 'juillet';
    8 : Result := 'aot';
    9 : Result := 'septembre';
    10 : Result := 'octobre';
    11 : Result := 'novembre';
    12 : Result := 'dcembre';
    else Result := '';
  end;
  if Maj and (Result <> '') then Result[1] := UpperCase(Result[1])[1];
end;

end.
