unit SjrdExtra;

interface

{$I SJRDUnits.inc}

uses
{$IFDEF SJRD_WINDOWS}
  Windows, Registry,
{$ENDIF}
  SysUtils, Classes, ZLib, SjrdUsual, SjrdResStrings;

type
  EUpdateResError = class(EAPIError) end platform;

function CorrectIdentifier(Ident : string) : boolean;
// Vrifie si Ident est un identificateur Delphi correct

function GetMethodFromName(Obj : TObject; MethodName : ShortString) : TMethod;
// Renvoie l'adresse de la mthode de nom NomMethode de l'objet Objet

function ConvertDoubleToInt64(Value : Double) : Int64;
// Convertit une valeur Double en la valeur Int64 ayant les mmes bits
// Attention ! Il n'y a aucune correspondance entre Value et Result
//             Cette fonction est totalement empirique

function ConvertInt64ToDouble(Value : Int64) : Double;
// Convertit une valeur Int64 en la valeur Double ayant les mmes bits
// Attention ! Il n'y a aucune correspondance entre Value et Result
//             Cette fonction est totalement empirique

function StrToStrRepres(Str : string) : string;
// Renvoie la reprsentation de Str en Pascal
// (si Str = C'est Nol !, renvoie 'C''est Nol !')

function StrRepresToStr(Str : string) : string;
// Renvoie la chane reprsente en Pascal par Str
// (si Str = 'A qui l''tour ?', renvoie A qui l'tour ?)

function CharToCharRepres(Charact : Char) : string;
// Renvoie la reprsentation de Charact en Pascal
// (si Charact = A, renvoie 'A' ; si Charact = Tab, renvoie #9)

function CharRepresToChar(Str : string) : Char;
// Renvoie le caractre reprsent en Pascal par Str
// (si Str = 'A', renvoie A ; si Str = #10, renvoie LineFeed)

function SetOfCharsToString(SetOfChars : TSetOfChars) : string;
// Renvoie la chane reprsentant l'ensemble SetOfChars (sans les [])

function StringToSetOfChars(Str : string) : TSetOfChars;
// Renvoie l'ensemble de caractres reprsent par Str (Str avec ou sans les [])

{$IFDEF SJRD_WINDOWS}
function BeginUpdateRes(FileName : string) : integer; platform;
procedure AddResource(ResHandle : integer; ResName : string; Resource : TStream;
                      ResType : string = 'RT_RCDATA'); platform;
procedure DelResource(ResHandle : integer; ResName : string); platform;
procedure EndUpdateRes(ResHandle : integer; Cancel : boolean = False); platform;
procedure AddResToFile(FileName, ResName : string; Resource : TStream;
                       ResType : string = 'RT_RCDATA'); platform;
procedure DelResInFile(FileName, ResName : string); platform;
{$ENDIF}

const
  clNoComp      = ZLib.clNone;    // pas de compression
  clFastestComp = ZLib.clFastest; // compression la plus rapide
  clDefaultComp = ZLib.clDefault; // compression par dfaut
  clMaxComp     = ZLib.clMax;     // compression maximale
procedure CompressStream(Stream : TStream; Dest : TStream = nil;
                         CompressionLevel : TCompressionLevel = clDefaultComp);
// Compresse le contenu de Stream dans Dest (si Dest = nil, dans Stream)
procedure DecompressStream(Stream : TStream; Dest : TStream = nil);
// Dcompresse le contenu de Stream dans Dest (si Dest = nil, dans Stream)

implementation

function CorrectIdentifier(Ident : string) : boolean;
var I : integer;
begin
  Result := False;
  // Si Ident est vide, ce n'est un indentificateur correct
  if Ident = '' then exit;
  // Si le premier caractre n'est pas alphabtique,
  // ce n'est pas un identificateur correct
  if not (Ident[1] in ['A'..'Z', '_', 'a'..'z']) then exit;
  // Si l'un des caractres suivant n'est pas alphanumrique,
  // ce n'est pas un identificateur correct
  for I := 2 to Length(Ident) do
    if not (Ident[I] in ['0'..'9', 'A'..'Z', '_', 'a'..'z']) then exit;
  // Dans les autres cas, a l'est
  Result := True;
end;

function GetMethodFromName(Obj : TObject; MethodName : ShortString) : TMethod;
begin
  Result.Code := Obj.MethodAddress(MethodName);
  Result.Data := Obj;
end;

function ConvertDoubleToInt64(Value : Double) : Int64;
type
  TypeDeTransition = packed record
    case integer of
    0 : (DblValue : Double);
    1 : (IntValue : Int64);
  end;
var VarDeTransition : TypeDeTransition;
begin
  // Ceci est totalement empirique, seuls les bits sont gaux (ce qui ne
  // correspond absolument pas  la valeur)
  VarDeTransition.DblValue := Value;
  Result := VarDeTransition.IntValue;
end;

function ConvertInt64ToDouble(Value : Int64) : Double;
type
  TypeDeTransition = packed record
    case integer of
    0 : (IntValue : Int64);
    1 : (DblValue : Double);
  end;
var VarDeTransition : TypeDeTransition;
begin
  // Ceci est totalement empirique, seuls les bits sont gaux (ce qui ne
  // correspond absolument pas  la valeur)
  VarDeTransition.IntValue := Value;
  Result := VarDeTransition.DblValue;
end;

function StrToStrRepres(Str : string) : string;
begin
  Result := AnsiQuotedStr(Str, '''');
end;

function StrRepresToStr(Str : string) : string;
begin
  Result := AnsiDequotedStr(Str, '''');
end;

function CharToCharRepres(Charact : Char) : string;
begin
  if Charact < #32 then Result := '#'+IntToStr(Byte(Charact)) else
  if Charact = '''' then Result := '''''''''' else
  Result := ''''+Charact+'''';
end;

function CharRepresToChar(Str : string) : Char;
begin
  try
    Str := Trim(Str);
    if Str = '' then raise EConvertError.Create('');
    case Str[1] of
      '#' :
      begin
        // Le rsultat est le caractre dont le code ASCII est l'entier
        // spcifi  la suite
        Delete(Str, 1, 1);
        Result := Chr(StrToInt(Str));
      end;
      '''' :
      begin
        case Length(Str) of
          // Si 3 caractres, le troisime doit tre ' et le deuxime
          // est le caractre rsultat
          3 : if Str[3] = '''' then Result := Str[2] else
            raise EConvertError.Create('');
          // Si 4 caractres, ce doit tre '''', auquel cas le caractre
          // retour est '
          4 : if Str = '''''''''' then Result := '''' else
            raise EConvertError.Create('');
          // Sinon, ce n'est pas un caractre correct
          else raise EConvertError.Create('');
        end;
      end;
      else raise EConvertError.Create('');
    end;
  except
    on Error : EConvertError do
      raise EConvertError.CreateFmt(UncorrectChar, [Str]);
  end;
end;

function SetOfCharsToString(SetOfChars : TSetOfChars) : string;
var I, From : Word;
begin
  Result := '';
  I := 0;
  // On cherche d'abord le premier caractre inclu
  while (I <= 255) and (not (Chr(I) in SetOfChars)) do inc(I);
  while I <= 255 do
  begin
    // Chr(I) est inclu
    From := I;
    // On cherche le caractre suivant qui n'est pas inclu
    while (I <= 255) and (Chr(I) in SetOfChars) do inc(I);
    // On teste I-From, soit le nombre de caractre conscutifs
    case I-From of
      // 1 : on ajoute simplement ce caractre
      1 : Result := Result+', '+CharToCharRepres(Chr(From));
      // 2 : on ajoute ces deux caractres spars par des virgules
      2 : Result := Result+', '+CharToCharRepres(Chr(From))+', '+CharToCharRepres(Chr(I-1));
      // 3+ : on ajoute les deux extrmes spars par ..
      else Result := Result+', '+CharToCharRepres(Chr(From))+'..'+CharToCharRepres(Chr(I-1));
    end;
    // on cherche le caractre suivant inclu
    repeat inc(I) until (I > 255) or (Chr(I) in SetOfChars);
  end;
  // On supprime les deux premiers caractres, car ce sont ', '
  Delete(Result, 1, 2);
end;

function StringToSetOfChars(Str : string) : TSetOfChars;
var I : integer;
  function GetCharAt : Char;
  // Renvoie le caractre  la position courante et augmente I en consquence
  // Fonctionne sur le mme principe que CharRepresToChar
  var From : integer;
  begin
    case Str[I] of
      '#' :
      begin
        From := I+1;
        repeat inc(I) until (I > Length(Str)) or (not (Str[I] in ['0'..'9']));
        Result := Chr(StrToInt(Copy(Str, From, I-From)));
      end;
      '''' :
      begin
        inc(I);
        if I > Length(Str) then
          raise EConvertError.Create('');
        if Str[I] = '''' then
        begin
          if I+2 > Length(Str) then
            raise EConvertError.Create('');
          if (Str[I+1] <> '''') or (Str[I+2] <> '''') then
            raise EConvertError.Create('');
          Result := '''';
          inc(I, 3);
        end else
        begin
          if I+1 > Length(Str) then
            raise EConvertError.Create('');
          if Str[I+1] <> '''' then
            raise EConvertError.Create('');
          Result := Str[I];
          inc(I, 2);
        end;
      end;
      else raise EConvertError.Create('');
    end;
  end;
var C1, C2 : Char;
begin
  try
    Result := [];
    Str := Trim(Str);
    // Si Str est vide, il n'y a aucun caractre dans l'ensemble
    if Str = '' then exit;
    // Si il y des [] aux extrmits, on les supprime
    if (Str[1] = '[') and (Str[Length(Str)] = ']') then
      Str := Trim(Copy(Str, 2, Length(Str)-2));

    I := 1;
    while I <= Length(Str) do
    begin
      // On rcupre le caractre  la position courante
      C1 := GetCharAt;
      // On passe tous les espaces
      while (I <= Length(Str)) and (Str[I] = ' ') do inc(I);

      // Si I > Length(Str), on ajoute le caractre et on arrte
      if I > Length(Str) then
      begin
        Include(Result, C1);
        Break;
      end;

      // Si Str[I] = ',', on ajoute le caractre et on passe la virgule
      if Str[I] = ',' then
      begin
        // On ajoute le caractre
        Include(Result, C1);
        // On passe la virgule et les espaces
        repeat inc(I) until (I > Length(Str)) or (Str[I] <> ' ');
        // Si on a atteint la fin de la chane, il y a une erreur
        // (on termine par une virgule)
        if I > Length(Str) then
          raise EConvertError.Create('');
        Continue;
      end;

      // Si Str[I] = '.', ce doit tre une plage de caractres
      if Str[I] = '.' then
      begin
        // On teste si le caractre suivant est aussi un point
        inc(I);
        if (I > Length(Str)) or (Str[I] <> '.') then
          raise EConvertError.Create('');
        // On passe ce point et les espaces
        repeat inc(I) until (I > Length(Str)) or (Str[I] <> ' ');
        // On rcupre le deuxime caractre
        C2 := GetCharAt;
        // On passe les espaces
        while (I <= Length(Str)) and (Str[I] = ' ') do inc(I);

        // Si I > Length(Str), on ajoute la plage de caractre et on termine
        if I > Length(Str) then
        begin
          Result := Result+[C1..C2];
          Break;
        end;

        // Si Str[I] = ',', on ajoute les caractres et on passe la virgule
        if Str[I] = ',' then
        begin
          // On ajoute la plage de caractres
          Result := Result+[C1..C2];
          // On passe la virgule et les espaces
          repeat inc(I) until (I > Length(Str)) or (Str[I] <> ' ');
          // Si on a atteint la fin de la chane, il y a une erreur
          // (on termine par une virgule)
          if I > Length(Str) then
            raise EConvertError.Create('');
          Continue;
        end;
        raise EConvertError.Create('');
      end;
      raise EConvertError.Create('');
    end;
  except
    on Error : EConvertError do
      raise EConvertError.CreateFmt(UncorrectSetOfChars, [Str]);
  end;
end;

///////////////////////////////////////
/// Ajout-suppression de ressources ///
///////////////////////////////////////
{$IFDEF SJRD_WINDOWS}

function BeginUpdateRes(FileName : string) : integer;
begin
  // Appel de Windows.BeginUpdateResource
  Result := BeginUpdateResource(PChar(FileName), False);
  // Si Result = 0, il y a eu une erreur API
  if Result = 0 then
    raise EUpdateResError.Create;
end;

procedure AddResource(ResHandle : integer; ResName : string; Resource : TStream; ResType : string = 'RT_RCDATA');
var MemRes : TMemoryStream;
    MustFreeRes, OK : boolean;
begin
  MustFreeRes := False;
  // On met dans MemRes un flux mmoire qui contient les donnes de la ressource
  if Resource is TMemoryStream then MemRes := Resource as TMemoryStream else
  begin
    MemRes := TMemoryStream.Create;
    MemRes.LoadFromStream(Resource);
    MustFreeRes := True;
  end;
  // Appel de Windows.UpdateResource
  OK := UpdateResource(ResHandle, PChar(ResType), PChar(ResName), 0,
                       MemRes.Memory, MemRes.Size);
  // On supprime le flux mmoire si on l'a cr
  if MustFreeRes then MemRes.Free;
  // Si UpdateResource a renvoy False, il y a eu une erreur
  if not OK then raise EUpdateResError.Create;
end;

procedure DelResource(ResHandle : integer; ResName : string);
begin
  // Appel de Windows.UpdateResource
  if not UpdateResource(ResHandle, '', PChar(ResName), 0, nil, 0) then
  // Si UpdateResource a renvoy False, il y a eu une erreur
    raise EUpdateResError.Create;
end;

procedure EndUpdateRes(ResHandle : integer; Cancel : boolean = False);
begin
  // Appel de Windows.EndUpdateResource
  if not EndUpdateResource(ResHandle, Cancel) then
  // Si EndUpdateResource a renvoy False, il y a eu une erreur
    raise EUpdateResError.Create;
end;

procedure AddResToFile(FileName, ResName : string; Resource : TStream; ResType : string = 'RT_RCDATA');
var ResHandle : integer;
begin
  ResHandle := BeginUpdateRes(FileName);
  try
    AddResource(ResHandle, ResName, Resource, ResType);
    EndUpdateRes(ResHandle);
  except
    try EndUpdateRes(ResHandle, True) except end;
    raise;
  end;
end;

procedure DelResInFile(FileName, ResName : string);
var ResHandle : integer;
begin
  ResHandle := BeginUpdateRes(FileName);
  try
    DelResource(ResHandle, ResName);
    EndUpdateRes(ResHandle);
  except
    try EndUpdateRes(ResHandle, True) except end;
    raise;
  end;
end;

{$ENDIF}

procedure CompressStream(Stream : TStream; Dest : TStream = nil;
                         CompressionLevel : TCompressionLevel = clDefaultComp);
var Compress : TCompressionStream;
    Destination : TStream;
begin
  // Si Dest vaut nil, on cre un flux temporaire de destination
  if Dest = nil then Destination := TMemoryStream.Create else
  begin
    Destination := Dest;
    Destination.Position := 0;
    Destination.Size := 0;
  end;
  // Cration, utilisation et libration du flux de compression
  Compress := TCompressionStream.Create(CompressionLevel, Destination);
  Compress.CopyFrom(Stream, 0);
  Compress.Free;
  // Si Dest vaut nil, on recopie Destination dans Stream
  if Dest = nil then
  begin
    Stream.Position := 0;
    Stream.Size := 0;
    Stream.CopyFrom(Destination, 0);
    Destination.Free;
  end;
end;

procedure DecompressStream(Stream : TStream; Dest : TStream = nil);
var Decompress : TDecompressionStream;
    Destination : TStream;
    Buffer : array [0..1023] of Byte;
    Copies : integer;
begin
  // Si Dest vaut nil, on cre un flux temporaire de destination
  if Dest = nil then Destination := TMemoryStream.Create else
  begin
    Destination := Dest;
    Destination.Position := 0;
    Destination.Size := 0;
  end;
  // Cration, utilisation et libration du flux de dcompression
  Decompress := TDecompressionStream.Create(Stream);
  Decompress.Position := 0;
  repeat
    Copies := Decompress.Read(Buffer, 1024);
    Destination.Write(Buffer, Copies);
  until Copies < 1024;
  Decompress.Free;
  // Si Dest vaut nil, on recopieDestination dans Stream
  if Dest = nil then
  begin
    Stream.Position := 0;
    Stream.Size := 0;
    Stream.CopyFrom(Destination, 0);
  end;
end;

end.
