unit SjrdLists;

interface

{$I SJRDUnits.inc}

uses
  SysUtils, Classes, SjrdUsual, SjrdStrUtils, SjrdResStrings;

type
  StringsOps = class
  // Propose une srie de mthodes pouvant tre applique  des instances de
  // TStrings (toutes les mthodes de recherche ignorent la diffrence
  // MAJ/min dans le cas d'un TStringList avec CaseSensitive  False)
  private
    class function StrEqual(Strings : TStrings; Str1, Str2 : string) : boolean;
  public
    class function  IndexOf      (Strings : TStrings; Str : string; BeginSearchAt : integer = 0; EndSearchAt : integer = -1) : integer;
      // Recherche la chane Str entre BeginSearchAt et EndSearchAt inclus
      // Si EndSearchAt = -1, la recherche s'effectue jusqu'au bout de la liste
    class function  FindText     (Strings : TStrings; Text : string; BeginSearchAt : integer = 0; EndSearchAt : integer = -1) : integer;
      // Recherche le texte Text entre BeginSearchAt et EndSearchAt inclus
      // Si EndSearchAt = -1, la recherche s'effectue jusqu'au bout de la liste
    class function  FindFirstWord(Strings : TStrings; Word : string; BeginSearchAt : integer = 0; EndSearchAt : integer = -1) : integer;
      // Recherche le mot Word entre BeginSearchAt et EndSearchAt inclus
      // mais uniquement en premire position (il peut y avoir des espaces devant)
      // Si EndSearchAt = -1, la recherche s'effectue jusqu'au bout de la liste
    class function  FindAtPos    (Strings : TStrings; SubStr : string; Position : integer = 1; BeginSearchAt : integer = 0; EndSearchAt : integer = -1) : integer;
      // Recherche la sous-chane SubStr  la position Position de chaque
      // chane de la liste entre BeginSearchAt et EndSearchAt inclus
      // Si EndSearchAt = -1, la recherche s'effectue jusqu'au bout de la liste
    class procedure CopyFrom     (Strings : TStrings; Source : TStrings; Index : integer = 0; Count : integer = -1);
      // Clear + AddFrom avec les mmes paramtres
    class procedure AddFrom      (Strings : TStrings; Source : TStrings; Index : integer = 0; Count : integer = -1);
      // Ajoute les Count chanes  partir de Index depuis Source  Strings
    class procedure FromString   (Strings : TStrings; Str, Delim : string; NotIn : string = '');
      // Clear + AddFromString avec les mmes paramtres
    class procedure AddFromString(Strings : TStrings; Str, Delim : string; NotIn : string = '');
      // Dcoupe Str en sous-chanes dlimites par Delim et les ajoute
      // Les chanes Delim se trouvant entre un caractre impair de NotIn et le
      // caractre pair lui correspondant ne sont pas pris en compte
  end;

  TSjrdStrings = class(TStringList)
  // Amlioration de TStringList pour lui ajouter les mthodes de StringsOps
  // ainsi qu'un index interne permettant de parcourir aisment toutes les
  // chanes dans l'ordre
  private
    FIndex : integer;
    function GetHasMoreString : boolean;
    procedure SetIndex(New : integer);
  public
    constructor Create;
    constructor CreateFromFile(FileName : TFileName);
    constructor CreateFromString(Str, Delim : string; NotIn : string = '');
    constructor CreateAssign(Source : TPersistent);

    // Les mthodes suivantes sont des appels aux mthodes correspondantes
    // de la classe StringsOps
    function  IndexOfEx    (Str : string; BeginSearchAt : integer = 0; EndSearchAt : integer = -1) : integer;
    function  FindText     (Text : string; BeginSearchAt : integer = 0; EndSearchAt : integer = -1) : integer;
    function  FindFirstWord(Word : string; BeginSearchAt : integer = 0; EndSearchAt : integer = -1) : integer;
    function  FindAtPos    (SubStr : string; Position : integer = 1; BeginSearchAt : integer = 0; EndSearchAt : integer = -1) : integer;
    procedure CopyFrom     (Source : TStrings; Index : integer = 0; Count : integer = -1);
    procedure AddFrom      (Source : TStrings; Index : integer = 0; Count : integer = -1);
    procedure FromString   (Str, Delim : string; NotIn : string = '');
    procedure AddFromString(Str, Delim : string; NotIn : string = '');

    // Gestion de l'index
    procedure Reset;
      // Remet l'index  0
    function NextString : string;
      // Renvoie la chane suivante (incrmente aussi l'index interne)
    property HasMoreString : boolean read GetHasMoreString;
      // True s'il y a encore une ou plusieurs chane(s), False sinon
    property Index : integer read FIndex write SetIndex;
      // Index
  end;

  TSjrdList = class;
  TSjrdListClass = class of TSjrdList;

  TSjrdList = class(TPersistent)
  // Classe de base pour les listes dont les lments sont de taille homogne
  // Ne crez pas d'instance de TSjrdList, mais crez plutt des instances de
  // ses classes descendantes
    // Ne pas utiliser pour des listes de chanes, de pointeurs ou d'objets ;
    // dans ces cas, utiliser respectivement
    // TStringList (Classes), TList (Classes) et TObjectList (Contnrs)
  private
    FStream : TMemoryStream;
    FItemSize : integer;

    function GetCount : integer;
    procedure SetCount(New : integer);
    function GetHasMoreValue : boolean;
    function GetIndex : integer;
    procedure SetIndex(New : integer);
  protected
    procedure DefineProperties(Filer : TFiler); override;

    procedure AssignTo(Dest : TPersistent); override;

    // Renvoie True si SjrdListClass est une classe d'assignation
    function IsAssignClass(SjrdListClass : TSjrdListClass) : boolean; virtual;

    // Les mthodes suivantes doivent tre appelles par les mthodes de mme
    // nom (sans le _) des descendants
    procedure _Read(var Buffer);
    procedure _Write(var Buffer);
    procedure _GetItems(AIndex : integer; var Buffer);
    procedure _SetItems(AIndex : integer; var Buffer);
    function _Add(var Buffer) : integer;
    function _Insert(AIndex : integer; var Buffer) : integer;
    procedure _Delete(AIndex : integer; var Buffer);

    property ItemSize : integer read FItemSize;
  public
    constructor Create(ItemSize : integer);
    destructor Destroy; override;

    procedure Assign(Source : TPersistent); override;
    procedure Clear;
    procedure Reset;

    procedure LoadFromStream(Stream : TStream);
    procedure SaveToStream(Stream : TStream);
    procedure LoadFromFile(FileName : TFileName);
    procedure SaveToFile(FileName : TFileName);

    property Count : integer read GetCount write SetCount;
    property HasMoreValue : boolean read GetHasMoreValue;
    property Index : integer read GetIndex write SetIndex;
  end;

  TIntegerList = class(TSjrdList)
  // Gre une liste d'entiers
  private
    function GetItems(Index : integer) : Int64;
    procedure SetItems(Index : integer; New : Int64);
    procedure MakeItGood(var Value : Int64);
  protected
    procedure AssignTo(Dest : TPersistent); override;

    function IsAssignClass(SjrdListClass : TSjrdListClass) : boolean; override;
  public
    constructor Create(IntSize : integer = 4);
    constructor CreateAssign(Source : TPersistent; IntSize : integer = 4);

    procedure Assign(Source : TPersistent); override;

    function Read : Int64;
    procedure Write(New : Int64);
    function Add(New : Int64) : integer;
    function Insert(Index : integer; New : Int64) : integer;
    function Delete(Index : integer) : Int64;

    property Items[index : integer] : Int64 read GetItems write SetItems; default;
  end;

  TUnsignedIntList = class(TSjrdList)
  // Gre une liste d'entiers non signs
  private
    function GetItems(Index : integer) : LongWord;
    procedure SetItems(Index : integer; New : LongWord);
    procedure MakeItGood(var Value : LongWord);
  protected
    procedure AssignTo(Dest : TPersistent); override;

    function IsAssignClass(SjrdListClass : TSjrdListClass) : boolean; override;
  public
    constructor Create(IntSize : integer = 4);
    constructor CreateAssign(Source : TPersistent; IntSize : integer = 4);

    procedure Assign(Source : TPersistent); override;

    function Read : LongWord;
    procedure Write(New : LongWord);
    function Add(New : LongWord) : integer;
    function Insert(Index : integer; New : LongWord) : integer;
    function Delete(Index : integer) : LongWord;

    property Items[index : integer] : LongWord read GetItems write SetItems; default;
  end;

  TExtendedList = class(TSjrdList)
  // Gre une liste de Extended
  private
    function GetItems(Index : integer) : Extended;
    procedure SetItems(Index : integer; New : Extended);
  protected
    procedure AssignTo(Dest : TPersistent); override;

    function IsAssignClass(SjrdListClass : TSjrdListClass) : boolean; override;
  public
    constructor Create;
    constructor CreateAssign(Source : TPersistent);

    procedure Assign(Source : TPersistent); override;

    function Read : Extended;
    procedure Write(New : Extended);
    function Add(New : Extended) : integer;
    function Insert(Index : integer; New : Extended) : integer;
    function Delete(Index : integer) : Extended;

    property Items[index : integer] : Extended read GetItems write SetItems; default;
  end;

var AppParams : TSjrdStrings;
// Contient la liste des paramtres de l'application sous forme de TSjrdStrings
// Vous pouvez y supprimer ou rajouter des lments : AppParams n'est utilis
// par aucune routine dans les Units SJRD

implementation

uses
  SjrdMaths;

/////////////////////////
/// Classe StringsOps ///
/////////////////////////

class function StringsOps.StrEqual(Strings : TStrings; Str1, Str2 : string) : boolean;
begin
  // Si Strings est une instance de TStringList et que sa proprit
  // CaseSensitive est  True, la comparaison se fait en respectant la casse.
  if (Strings is TStringList) and TStringList(Strings).CaseSensitive then
    Result := CompareStr(Str1, Str2) = 0
  // Dans tous les autres cas, la comparaison se fait sans respecter la casse.
  else
    Result := CompareText(Str1, Str2) = 0;
end;

class function StringsOps.IndexOf(Strings : TStrings; Str : string; BeginSearchAt : integer = 0; EndSearchAt : integer = -1) : integer;
var Strs : TStrings;
begin
  Strs := Strings;
  with Strings do
  begin
    // On vrifie que BeginSearchAt et EndSearchAt sont des entres correctes
    if BeginSearchAt < 0 then BeginSearchAt := 0;
    if (EndSearchAt < 0) or (EndSearchAt >= Count) then EndSearchAt := Count-1;
    Result := BeginSearchAt; // On commence la recherche  BeginSearchAt
    while Result <= EndSearchAt do // On termine  EndSearchAt
      if StrEqual(Strs, Str, Strings[BeginSearchAt]) then exit else
        inc(BeginSearchAt);
    Result := -1;
  end;
end;

class function StringsOps.FindText(Strings : TStrings; Text : string; BeginSearchAt : integer = 0; EndSearchAt : integer = -1) : integer;
var I, Len : integer;
    Strs : TStrings;
begin
  Strs := Strings;
  with Strings do
  begin
    // On vrifie que BeginSearchAt et EndSearchAt sont des entres correctes
    if BeginSearchAt < 0 then BeginSearchAt := 0;
    if (EndSearchAt < 0) or (EndSearchAt >= Count) then EndSearchAt := Count-1;
    Len := Length(Text);
    Result := BeginSearchAt; // On commence la recherche  BeginSearchAt
    while Result <= EndSearchAt do // On termine  EndSearchAt
    begin
      for I := 1 to Length(Strings[Result])-Len+1 do
        if StrEqual(Strs, Text, Copy(Strings[Result], I, Len)) then exit;
      inc(Result);
    end;
    Result := -1;
  end;
end;

class function StringsOps.FindFirstWord(Strings : TStrings; Word : string; BeginSearchAt : integer = 0; EndSearchAt : integer = -1) : integer;
var Strs : TStrings;
begin
  Strs := Strings;
  with Strings do
  begin
    // On vrifie que BeginSearchAt et EndSearchAt sont des entres correctes
    if BeginSearchAt < 0 then BeginSearchAt := 0;
    if (EndSearchAt < 0) or (EndSearchAt >= Count) then EndSearchAt := Count-1;
    Result := BeginSearchAt; // On commence la recherche  BeginSearchAt
    while Result <= EndSearchAt do // On termine  EndSearchAt
      if StrEqual(Strs, Word, GetXWord(Trim(Strings[Result]), 1)) then exit else
        inc(Result);
    Result := -1;
  end;
end;

class function StringsOps.FindAtPos(Strings : TStrings; SubStr : string; Position : integer = 1; BeginSearchAt : integer = 0; EndSearchAt : integer = -1) : integer;
var Len : integer;
    Strs : TStrings;
begin
  Strs := Strings;
  with Strings do
  begin
    // On vrifie que BeginSearchAt et EndSearchAt sont des entres correctes
    if BeginSearchAt < 0 then BeginSearchAt := 0;
    if (EndSearchAt < 0) or (EndSearchAt >= Count) then EndSearchAt := Count-1;
    Len := Length(SubStr);
    Result := BeginSearchAt; // On commence la recherche  BeginSearchAt
    while Result <= EndSearchAt do // On termine  EndSearchAt
      if StrEqual(Strs, SubStr, Copy(Strings[Result], Position, Len)) then exit else
        inc(Result);
  end;
end;

class procedure StringsOps.CopyFrom(Strings : TStrings; Source : TStrings; Index : integer = 0; Count : integer = -1);
begin
  Strings.Clear;
  AddFrom(Strings, Source, Index, Count);
end;

class procedure StringsOps.AddFrom(Strings : TStrings; Source : TStrings; Index : integer = 0; Count : integer = -1);
var I, EndAt : integer;
begin
  // On vrifie que Index et Count sont des entres correctes
  if Index < 0 then exit;
  if (Count < 0) or (Index+Count > Source.Count) then EndAt := Source.Count-1 else
    EndAt := Index+Count-1;
  // On recopie les chanes
  for I := Index to EndAt do
    Strings.Append(Source[I]);
end;

class procedure StringsOps.FromString(Strings : TStrings; Str, Delim : string; NotIn : string = '');
begin
  Strings.Clear;
  AddFromString(Strings, Str, Delim, NotIn);
end;

class procedure StringsOps.AddFromString(Strings : TStrings; Str, Delim : string; NotIn : string = '');
var I, J, Len : integer;
    Chaine, NotIn2 : string;
    C : Char;
begin
  with Strings do
  begin
    // On vrifie que NotIn contient un nombre pair de caractres
    if (Length(NotIn) mod 2) = 1 then
      raise Exception.Create(NotInMustPairsOfChars);
    // On vrifie qu'il n'y a pas d'interfrence entre Delim et NotIn
    for I := 1 to Length(NotIn) do if Pos(NotIn[I], Delim) > 0 then
      raise Exception.Create(DelimMustDifferentThanNotIn);

    Chaine := Str; // On passe la chane  trater dans Chaine
    Len := Length(Chaine);
    // Sparation de NotIn en NotIn et NotIn2
    Str := NotIn;
    NotIn := '';
    NotIn2 := '';
    for I := 1 to Length(Str) do if (I mod 2) = 1 then
      NotIn := NotIn+Str[I] else NotIn2 := NotIn2+Str[I];

    I := 1;
    while True do
    begin
      // On boucle jusqu' trouver un caractre qui n'est pas dans Delim
      // On ignore de ce fait plusieurs caractres de Delim  la suite
      while (I <= Len) and (Pos(Chaine[I], Delim) <> 0) do inc(I);
      if (I > Len) then Break;
      // On recherche le caractre de Delim suivant
      J := I;
      while (J <= Len) and (Pos(Chaine[J], Delim) = 0) do
      begin
        // Si on trouve un caractre de NotIn, on boucle jusqu' trouver le
        // caractre correspondant de NotIn2
        if Pos(Chaine[J], NotIn) > 0 then
        begin
          C := NotIn2[Pos(Chaine[J], NotIn)];
          inc(J);
          while (J <= Len) and (Chaine[J] <> C) do inc(J);
        end;
        inc(J);
      end;
      // On ajoute la chane repre par les caractres de Delim
      Add(Copy(Chaine, I, J-I));
      I := J;
    end;
  end;
end;

///////////////////////////////////////////////
/// Procdures et fonctions de TSjrdStrings ///
///////////////////////////////////////////////

constructor TSjrdStrings.Create;
begin
  inherited Create;
  FIndex := 0;
end;

constructor TSjrdStrings.CreateFromFile(FileName : TFileName);
begin
  Create;
  LoadFromFile(FileName);
end;

constructor TSjrdStrings.CreateFromString(Str, Delim : string; NotIn : string = '');
begin
  Create;
  FromString(Str, Delim, NotIn);
end;

constructor TSjrdStrings.CreateAssign(Source : TPersistent);
begin
  Create;
  Assign(Source);
end;

function TSjrdStrings.GetHasMoreString : boolean;
begin
  Result := Index < Count;
end;

procedure TSjrdStrings.SetIndex(New : integer);
begin
  if New >= 0 then FIndex := New;
end;

function TSjrdStrings.IndexOfEx(Str : string; BeginSearchAt : integer = 0; EndSearchAt : integer = -1) : integer;
begin
  Result := StringsOps.IndexOf(Self, Str, BeginSearchAt, EndSearchAt);
end;

function TSjrdStrings.FindText(Text : string; BeginSearchAt : integer = 0; EndSearchAt : integer = -1) : integer;
begin
  Result := StringsOps.FindText(Self, Text, BeginSearchAt, EndSearchAt);
end;

function TSjrdStrings.FindFirstWord(Word : string; BeginSearchAt : integer = 0; EndSearchAt : integer = -1) : integer;
begin
  Result := StringsOps.FindFirstWord(Self, Word, BeginSearchAt, EndSearchAt);
end;

function TSjrdStrings.FindAtPos(SubStr : string; Position : integer = 1; BeginSearchAt : integer = 0; EndSearchAt : integer = -1) : integer;
begin
  Result := StringsOps.FindAtPos(Self, SubStr, Position, BeginSearchAt, EndSearchAt);
end;

procedure TSjrdStrings.CopyFrom(Source : TStrings; Index : integer = 0; Count : integer = -1);
begin
  StringsOps.CopyFrom(Self, Source, Index, Count);
end;

procedure TSjrdStrings.AddFrom(Source : TStrings; Index : integer = 0; Count : integer = -1);
begin
  StringsOps.AddFrom(Self, Source, Index, Count);
end;

procedure TSjrdStrings.FromString(Str, Delim : string; NotIn : string = '');
begin
  StringsOps.FromString(Self, Str, Delim, NotIn);
end;

procedure TSjrdStrings.AddFromString(Str, Delim : string; NotIn : string = '');
begin
  StringsOps.AddFromString(Self, Str, Delim, NotIn);
end;

procedure TSjrdStrings.Reset;
begin
  FIndex := 0;
end;

function TSjrdStrings.NextString : string;
begin
  if HasMoreString then Result := Strings[Index] else Result := '';
  inc(FIndex);
end;

////////////////////////
/// Classe TSjrdList ///
////////////////////////

constructor TSjrdList.Create(ItemSize : integer);
begin
  FStream := TMemoryStream.Create;
  FItemSize := ItemSize;
end;

destructor TSjrdList.Destroy;
begin
  FStream.Free;
  inherited Destroy;
end;

function TSjrdList.GetCount : integer;
begin
  // Le nombre d'lments est la taille du flux divises la taille d'un lment
  Result := FStream.Size div FItemSize;
end;

procedure TSjrdList.SetCount(New : integer);
begin
  // La taille du flux est le nombre d'lments multipli par leur taille
  if New <= 0 then FStream.SetSize(0) else
    FStream.SetSize(New*FItemSize);
end;

function TSjrdList.GetHasMoreValue : boolean;
begin
  // Il y a encore une ou plusieurs valeurs  lire ssi la position du flux
  // est infrieure  sa taille
  Result := FStream.Position < FStream.Size;
end;

function TSjrdList.GetIndex : integer;
begin
  // L'index est la position du flux divis par la taille des lments
  Result := FStream.Position div FItemSize;
end;

procedure TSjrdList.SetIndex(New : integer);
begin
  // On vrifie que New est bien un index correct
  if (New < 0) or (New > Count) then
    raise EListError.CreateFmt(IndexOutOfRange, [New]);
  // La position du flux est l'index multipli par la taille des lments
  FStream.Position := New*FItemSize;
end;

procedure TSjrdList.DefineProperties(Filer : TFiler);
begin
  inherited;
  // La ligne suivante cre une proprit publie inexistante qui permet
  // d'enregistrer le contenu via WriteComponent et de le lire via
  // ReadComponent
  Filer.DefineBinaryProperty('Items', LoadFromStream, SaveToStream, True);
end;

procedure TSjrdList.AssignTo(Dest : TPersistent);
begin
  // On autorise l'assignation ssi Dest.ClassType fait partie de la liste des
  // Classes d'assignation et que les tailles d'lements sont les mmes
  if (Dest is TSjrdList) and
     (TSjrdList(Dest).FItemSize = FItemSize) and
     IsAssignClass(TSjrdListClass(Dest.ClassType)) then
  begin
    TSjrdList(Dest).FStream.LoadFromStream(FStream);
  end else
  inherited;
end;

function TSjrdList.IsAssignClass(SjrdListClass : TSjrdListClass) : boolean;
begin
  // Si on remonte jusqu'ici, c'est que ce n'est une classe d'assignation
  Result := False;
end;

procedure TSjrdList._Read(var Buffer);
begin
  // Lit un lment  la position courante
  FStream.ReadBuffer(Buffer, FItemSize);
end;

procedure TSjrdList._Write(var Buffer);
begin
  // crit l'lment  la position courante
  FStream.WriteBuffer(Buffer, FItemSize);
end;

procedure TSjrdList._GetItems(AIndex : integer; var Buffer);
begin
  // On vrifie que AIndex est un index correct
  if (AIndex < 0) or (AIndex >= Count) then
    raise EListError.CreateFmt(IndexOutOfRange, [AIndex]);
  // On se place au bon endroit pour lire l'lment
  Index := AIndex;
  // On lit l'lment
  _Read(Buffer);
end;

procedure TSjrdList._SetItems(AIndex : integer; var Buffer);
begin
  // On vrifie que AIndex est un index correct
  if (AIndex < 0) or (AIndex >= Count) then
    raise EListError.CreateFmt(IndexOutOfRange, [AIndex]);
  // On se place au bon endroit pour crire l'lment
  Index := AIndex;
  // On crit l'lment
  _Write(Buffer);
end;

function TSjrdList._Add(var Buffer) : integer;
begin
  // Le rsultat est l'index du nouvel lment, soit le nombre d'lments actuel
  Result := Count;
  // On se place  la fin du flux
  Index := Result;
  // On crit l'lment ; comme c'est un flux mmoire, cela agrandira
  // automatiquemnt le flux de faon  accepter les nouvelles donnes
  _Write(Buffer);
end;

function TSjrdList._Insert(AIndex : integer; var Buffer) : integer;
var Temp : TMemoryStream;
begin
  // On vrifie que AIndex est un index correct
  if (AIndex < 0) or (AIndex > Count) then
    raise EListError.CreateFmt(IndexOutOfRange, [AIndex]);

  // Si AIndex vaut Count, on appelle _Add, sinon, on effectue le traitement
  if AIndex = Count then Result := _Add(Buffer) else
  begin
    Temp := TMemoryStream.Create;
    Result := AIndex; // La valeur de retour est l'index du nouvel lment

    // On copie tous les lments  partir de AIndex dans Temp
    Index := AIndex;
    Temp.CopyFrom(FStream, FStream.Size-FStream.Position);

    // On agrandi la liste
    Count := Count+1;

    // On crit le nouvel lment et  sa suite le contenu de Temp
    Index := AIndex;
    _Write(Buffer);
    FStream.CopyFrom(Temp, 0);

    Temp.Free;
  end;
end;

procedure TSjrdList._Delete(AIndex : integer; var Buffer);
var Temp : TMemoryStream;
begin
  // On vrifie que AIndex est un index correct
  if (AIndex < 0) or (AIndex >= Count) then
    raise EListError.CreateFmt(IndexOutOfRange, [AIndex]);

  Temp := TMemoryStream.Create;

  // On lit la valeur de retour  l'emplacement qui va tre supprim
  Index := AIndex;
  _Read(Buffer);

  // On copie tous les lments aprs celui-l dans Temp
  Temp.CopyFrom(FStream, FStream.Size-FStream.Position);

  // On rduit la liste
  Count := Count-1;

  // On recopie le contenu de Temp  partir de l'emplacement qui a t supprim
  Index := AIndex;
  FStream.CopyFrom(Temp, 0);

  Temp.Free;
end;

procedure TSjrdList.Assign(Source : TPersistent);
begin
  // On autorise l'assignation ssi Source.ClassType fait partie de la liste des
  // classes d'assignation et que les tailles d'lements sont les mmes
  if (Source is TSjrdList) and
     (TSjrdList(Source).FItemSize = FItemSize) and
     IsAssignClass(TSjrdListClass(Source.ClassType)) then
  begin
    FStream.LoadFromStream(TSjrdList(Source).FStream);
  end else
  inherited;
end;

procedure TSjrdList.Clear;
begin
  Count := 0;
end;

procedure TSjrdList.Reset;
begin
  Index := 0;
end;

procedure TSjrdList.LoadFromStream(Stream : TStream);
var Size : Int64;
begin
  // On lit la taille sur 8 octets (Int64)
  Stream.Read(Size, 8);
  FStream.Size := 0;
  // Si la taille est plus grande que 0, on fait un CopyFrom
  if Size > 0 then
    FStream.CopyFrom(Stream, Size);
end;

procedure TSjrdList.SaveToStream(Stream : TStream);
var Size : Int64;
begin
  // On crit la taille sur 8 octets (Int64)
  Size := FStream.Size;
  Stream.Write(Size, 8);
  // Si la taille est plus grande que 0, on fait un CopyFrom
  if Size > 0 then
    Stream.CopyFrom(FStream, 0);
end;

procedure TSjrdList.LoadFromFile(FileName : TFileName);
var FileStream : TFileStream;
begin
  FileStream := TFileStream.Create(FileName, fmOpenRead or fmShareDenyNone);
  try
    LoadFromStream(FileStream);
  finally
    FileStream.Free;
  end;
end;

procedure TSjrdList.SaveToFile(FileName : TFileName);
var FileStream : TFileStream;
begin
  FileStream := TFileStream.Create(FileName, fmCreate or fmShareDenyNone);
  try
    SaveToStream(FileStream);
  finally
    FileStream.Free;
  end;
end;

///////////////////////////
/// Classe TIntegerList ///
///////////////////////////

constructor TIntegerList.Create(IntSize : integer = 4);
begin
  // On vrifie que IntSize est entre 1 et 8 inclus
  if (IntSize < 1) or (IntSize > 8) then
    raise ECreateError.CreateFmt(UncorrectIntSize, [IntSize]);
  inherited Create(IntSize);
end;

constructor TIntegerList.CreateAssign(Source : TPersistent; IntSize : integer = 4);
begin
  Create(IntSize);
  Assign(Source);
end;

function TIntegerList.GetItems(Index : integer) : Int64;
begin
  _GetItems(Index, Result);
  MakeItGood(Result);
end;

procedure TIntegerList.SetItems(Index : integer; New : Int64);
begin
  _SetItems(Index, New);
end;

procedure TIntegerList.MakeItGood(var Value : Int64);
// Cette procdure remplis les octets non stocks de manire  transformer
// un entier sur un nombre quelconque d'octets en Int64 (sur 8 octets)
type
  TRecVal = record
    case integer of
      1 : (Int : Int64);
      2 : (Ints : array[1..8] of Shortint);
  end;
var RecVal : TRecVal;
    I : integer;
    Remplis : Shortint;
begin
  RecVal.Int := Value;
    // On initialise RecVal.Int (et donc aussi RecVal.Ints)
  if RecVal.Ints[ItemSize] < 0 then Remplis := -1 else Remplis := 0;
    // Si le Shortint  la position ItemSize (c--d l'octet de poids le plus
    // fort stock) est ngatif (c--d si le nombre complet est ngatif),
    // on remplit les suivant avec -1 (11111111b) et sinon avec 0 (00000000b)
  for I := ItemSize+1 to 8 do RecVal.Ints[I] := Remplis;
    // On remplit les les octets non stocks avec la valeur de Remplis
  Value := RecVal.Int;
    // On ractualise Value
end;

procedure TIntegerList.AssignTo(Dest : TPersistent);
var DestStrings : TStrings;
begin
  // Si Dest est un TStrings on le remplit avec les formes chanes des lments
  if Dest is TStrings then
  begin
    DestStrings := TStrings(Dest);
    DestStrings.Clear;
    // Le principe "Reset-HasMoreValue-Read" tant plus rapide que
    // "for 0 to Count-1" avec TSjrdList, on l'utilise pour remplire le TStrings
    Reset;
    while HasMoreValue do
      DestStrings.Append(IntToStr(Read));
  end else
  inherited;
end;

function TIntegerList.IsAssignClass(SjrdListClass : TSjrdListClass) : boolean;
begin
  if SjrdListClass.InheritsFrom(TIntegerList) then Result := True else
  Result := inherited IsAssignClass(SjrdListClass);
end;

procedure TIntegerList.Assign(Source : TPersistent);
var SourceStrings : TStrings;
    I : integer;
begin
  // Si Source est un TStrings, on convertit les chanes en entiers
  if Source is TStrings then
  begin
    SourceStrings := TStrings(Source);
    Clear;
    for I := 0 to SourceStrings.Count-1 do
      Add(StrToInt64(SourceStrings[I]));
  end else
  inherited;
end;

function TIntegerList.Read : Int64;
begin
  _Read(Result);
  MakeItGood(Result);
end;

procedure TIntegerList.Write(New : Int64);
begin
  _Write(New);
end;

function TIntegerList.Add(New : Int64) : integer;
begin
  Result := _Add(New);
end;

function TIntegerList.Insert(Index : integer; New : Int64) : integer;
begin
  Result := _Insert(Index, New);
end;

function TIntegerList.Delete(Index : integer) : Int64;
begin
  _Delete(Index, Result);
  MakeItGood(Result);
end;

///////////////////////////////
/// Classe TUnsignedIntList ///
///////////////////////////////

constructor TUnsignedIntList.Create(IntSize : integer = 4);
begin
  // On vrifie que IntSize est entre 1 et 4 inclus
  if (IntSize < 1) or (IntSize > 4) then
    raise ECreateError.CreateFmt(UncorrectIntSize, [IntSize]);
  inherited Create(IntSize);
end;

constructor TUnsignedIntList.CreateAssign(Source : TPersistent; IntSize : integer = 4);
begin
  Create(IntSize);
  Assign(Source);
end;

function TUnsignedIntList.GetItems(Index : integer) : LongWord;
begin
  _GetItems(Index, Result);
  MakeItGood(Result);
end;

procedure TUnsignedIntList.SetItems(Index : integer; New : LongWord);
begin
  _SetItems(Index, New);
end;

procedure TUnsignedIntList.MakeItGood(var Value : LongWord);
// Cette procdure remplis les octets non stocks de manire  transformer
// un entier sur un nombre quelconque d'octets en LongWord (sur 4 octets)
type
  TRecVal = record
    case integer of
      1 : (Int : LongWord);
      2 : (Ints : array[1..4] of Byte);
  end;
var RecVal : TRecVal;
    I : integer;
begin
  RecVal.Int := Value;
    // On initialise RecVal.Int (et donc aussi RecVal.Ints)
  for I := ItemSize+1 to 4 do RecVal.Ints[I] := 0;
    // On remplit les octets non stocks avec des 0
  Value := RecVal.Int;
    // On ractualise Value
end;

procedure TUnsignedIntList.AssignTo(Dest : TPersistent);
var DestStrings : TStrings;
begin
  // Si Dest est un TStrings on le remplit avec les formes chanes des lments
  if Dest is TStrings then
  begin
    DestStrings := TStrings(Dest);
    DestStrings.Clear;
    // Le principe "Reset-HasMoreValue-Read" tant plus rapide que
    // "for 0 to Count-1" avec TSjrdList, on l'utilise pour remplire le TStrings
    Reset;
    while HasMoreValue do
      DestStrings.Append(IntToStr(Read));
  end else
  inherited;
end;

function TUnsignedIntList.IsAssignClass(SjrdListClass : TSjrdListClass) : boolean;
begin
  if SjrdListClass.InheritsFrom(TUnsignedIntList) then Result := True else
  Result := inherited IsAssignClass(SjrdListClass);
end;

procedure TUnsignedIntList.Assign(Source : TPersistent);
var SourceStrings : TStrings;
    I : integer;
    Val : Int64;
begin
  // Si Source est un TStrings, on convertit les chanes en entiers
  if Source is TStrings then
  begin
    SourceStrings := TStrings(Source);
    Clear;
    for I := 0 to SourceStrings.Count-1 do
    begin
      Val := StrToInt64(SourceStrings[I]);
      // On vrifie que Val est bien un LongWord
      if (Val < 0) or (Val > High(LongWord)) then
        raise EConvertError.CreateFmt(UncorrectLongWord, [SourceStrings[I]]);
      Add(Val);
    end;
  end else
  inherited;
end;

function TUnsignedIntList.Read : LongWord;
begin
  _Read(Result);
  MakeItGood(Result);
end;

procedure TUnsignedIntList.Write(New : LongWord);
begin
  _Write(New);
end;

function TUnsignedIntList.Add(New : LongWord) : integer;
begin
  Result := _Add(New);
end;

function TUnsignedIntList.Insert(Index : integer; New : LongWord) : integer;
begin
  Result := _Insert(Index, New);
end;

function TUnsignedIntList.Delete(Index : integer) : LongWord;
begin
  _Delete(Index, Result);
  MakeItGood(Result);
end;

////////////////////////////
/// Classe TExtendedList ///
////////////////////////////

constructor TExtendedList.Create;
begin
  inherited Create(sizeof(Extended));
end;

constructor TExtendedList.CreateAssign(Source : TPersistent);
begin
  Create;
  Assign(Source);
end;

function TExtendedList.GetItems(Index : integer) : Extended;
begin
  _GetItems(Index, Result);
end;

procedure TExtendedList.SetItems(Index : integer; New : Extended);
begin
  _SetItems(Index, New);
end;

procedure TExtendedList.AssignTo(Dest : TPersistent);
var DestStrings : TStrings;
begin
  // Si Dest est un TStrings on le remplit avec les formes chanes des lments
  if Dest is TStrings then
  begin
    DestStrings := TStrings(Dest);
    DestStrings.Clear;
    // Le principe "Reset-HasMoreValue-Read" tant plus rapide que
    // "for 0 to Count-1" avec TSjrdList, on l'utilise pour remplire le TStrings
    Reset;
    while HasMoreValue do
      DestStrings.Append(FloatToStr(Read));
  end else
  inherited;
end;

function TExtendedList.IsAssignClass(SjrdListClass : TSjrdListClass) : boolean;
begin
  if SjrdListClass.InheritsFrom(TExtendedList) then Result := True else
  Result := inherited IsAssignClass(SjrdListClass);
end;

procedure TExtendedList.Assign(Source : TPersistent);
var SourceStrings : TStrings;
    I : integer;
begin
  // Si Source est un TStrings, on convertit les chanes en nombres flottants
  if Source is TStrings then
  begin
    Clear;
    SourceStrings := TStrings(Source);
    for I := 0 to SourceStrings.Count-1 do
      Write(StrToFloat(SourceStrings[I]));
  end else
  inherited;
end;

function TExtendedList.Read : Extended;
begin
  _Read(Result);
end;

procedure TExtendedList.Write(New : Extended);
begin
  _Write(New);
end;

function TExtendedList.Add(New : Extended) : integer;
begin
  Result := _Add(New);
end;

function TExtendedList.Insert(Index : integer; New : Extended) : integer;
begin
  Result := _Insert(Index, New);
end;

function TExtendedList.Delete(Index : integer) : Extended;
begin
  _Delete(Index, Result);
end;

//////////////////////////////////////
/// Initialization et Finalization ///
//////////////////////////////////////

var I : integer;

initialization
  AppParams := TSjrdStrings.Create;
  // On remplit AppParams avec les paramtres envoys  l'application
  for I := 1 to ParamCount do AppParams.Append(ParamStr(I));
finalization
  AppParams.Free;
end.
