unit SjrdStrUtils;

interface

{$I SJRDUnits.inc}

{$IFDEF SJRD_WINDOWS}
uses
  Windows;
{$ENDIF}

type
  TSjrdCompareOption = (
    coIgnoreCase,        // Ignore la casse
    coIgnoreNonSpace,    // Ignore les accents
    coIgnoreSymbols);    // Ignore les symboles
  TSjrdCompareOptions = set of TSjrdCompareOption;

function NberSubStr(SubStr, Str : string) : integer;
// Renvoie le nombre d'occurences de SubStr dans Str
// Attention ! Si vous recherchez '..' dans '...', vous en trouverez 2 !

{$IFDEF SJRD_WINDOWS}
function SjrdCompareString(S1, S2 : string;
                           CompareOptions : TSjrdCompareOptions = []) : integer;
                           platform;
// Compare deux chanes suivant les options spcifies par CompareOptions
{$ENDIF}

function GetFirstToken(S : string; Token : Char) : string;
// Renvoie la premire sous-chane de S dlimite par Token
// (si Token n'est pas dans S, renvoie S)
function GetLastToken(S : string; Token : Char) : string;
// Renvoie la dernire sous-chane de S dlimite par Token
// (si Token n'est pas dans S, renvoie S)
function GetXToken(S : string; Token : Char; X : integer) : string;
// Renvoie la Xme sous-chane de S dlimite par Token

function GetXWord(S : string; X : integer) : string;
// Recherche le Xme mot dans S
// Attention ! L'appel GetXWord('Je m''appelle Sbastien', 2) renverra 'm''appelle' !

function PosWord(Wrd, Str : string; Index : integer = 1) : integer;
// Renvoie la premire position de Wrd dans Str  partir de la position Index
// Renvoie 0 si le mot n'est pas trouv
// Attention ! L'appel PosWord('Chose', 'Chose-Machin Chouette Chose') renverra 23
//                                                             *
// De mme que l'appel PosWord('revoir', 'Au revoir.') renverra 0 (non trouv  cause du '.')

implementation

uses
  StrUtils;

function NberSubStr(SubStr, Str : string) : integer;
var I : integer;
begin
  Result := 0;
  // On parcourt la chane et on incrmente Result si la correspondance est
  // tablie
  for I := 1 to Length(Str)-Length(SubStr)+1 do
    if Copy(Str, I, Length(SubStr)) = SubStr then inc(Result);
end;

{$IFDEF SJRD_WINDOWS}
function SjrdCompareString(S1, S2 : string;
                           CompareOptions : TSjrdCompareOptions = []) : integer;
var Flags : DWord;
begin
  Flags := 0;
  // On ajoute les flags de comparaison
  if coIgnoreCase     in CompareOptions then Flags := Flags+NORM_IGNORECASE;
  if coIgnoreNonSpace in CompareOptions then Flags := Flags+NORM_IGNORENONSPACE;
  if coIgnoreSymbols  in CompareOptions then Flags := Flags+NORM_IGNORESYMBOLS;
  // Appel de Windows.CompareString
  Result := Windows.CompareString(LOCALE_USER_DEFAULT, Flags,
                                  PChar(S1), -1, PChar(S2), -1)-2;
end;
{$ENDIF}

function GetFirstToken(S : string; Token : Char) : string;
var I : integer;
begin
  I := 1;
  // On parcourt la chane jusqu' trouver un caractre Token
  while (I <= Length(S)) and (S[I] <> Token) do inc(I);
  // On copie la chane depuis le dbut jusqu'au caractre avant Token
  Result := Copy(S, 1, I-1);
end;

function GetLastToken(S : string; Token : Char) : string;
var I : integer;
begin
  I := Length(S);
  // On parcourt la chane  l'envers jusqu' trouver un caractre Token
  while (I > 0) and (S[I] <> Token) do dec(I);
  // On copie la chane depuis le caractre aprs Token jusqu' la fin
  Result := Copy(S, I+1, Length(S));
end;

function GetXToken(S : string; Token : Char; X : integer) : string;
var I, J : integer;
begin
  dec(X);
  I := 1;
  // On boucle jusqu' trouver la bonne occurence de Token
  while (X > 0) and (I <= Length(S)) do
  begin
    if S[I] = Token then dec(X);
    inc(I);
  end;
  // Si X est encore plus grand que 0, c'est qu'il n'y a pas assez d'occurences
  if X > 0 then Result := '' else
  begin
    J := I;
    while (J <= Length(S)) and (S[J] <> Token) do inc(J);
    Result := Copy(S, I, J-I);
  end;
end;

function GetXWord(S : string; X : integer) : string;
begin
  Result := GetXToken(S, ' ', X);
end;

function PosWord(Wrd, Str : string; Index : integer = 1) : integer;
begin
  Result := PosEx(' '+Wrd+' ', ' '+Str+' ', Index);
end;

end.
