unit SjrdUsual;

interface

{$I SJRDUnits.inc}

uses
{$IFDEF SJRD_WINDOWS}
  Windows, Dialogs, Graphics, Registry, ShellAPI, MMSystem, ComObj, ShlObj,
  ActiveX,
{$ENDIF}
{$IFDEF SJRD_LINUX}
  QDialogs, QGraphics,
{$ENDIF}
  SysUtils, Classes, SjrdStrUtils, Forms;

type
  ECreateError = class(Exception);

  EAPIError = class(Exception)
  private
    FErrorCode : integer;
  public
    constructor Create(Error : integer); overload;
    constructor Create; overload;
    property ErrorCode : integer read FErrorCode;
  end platform;

  T3DPoint = record
    X, Y, Z : integer;
  end;

// Types pour ShowDialog
  // Dans le cas de Windows, ce sont des nouveaux types :
  {$IFDEF SJRD_WINDOWS}
    TDialogType = (dtCustom, dtInformation, dtWarning, dtError, dtConfirmation);
    TDialogButtons = (dbOK, dbOKCancel, dbYesNo, dbYesNoCancel, dbRetryCancel,
                      dbAbortRetryIgnore);
    TDialogResult = (drOK, drYes, drNo, drCancel, drAbort, drRetry, drIgnore);
  {$ENDIF}
  // Dans le cas de Linux, ce sont des constantes quivalentes aux valeurs
  // respective de MessageDlg.
  {$IFDEF SJRD_LINUX}
    TDialogType = type TMsgDlgType;
      dtCustom = mtCustom;
      dtInformation = mtInformation;
      dtWarning = mtWarning;
      dtError = mtError;
      dtConfirmation = mtConfirmation;
    TDialogButtons = type TMsgDlgButtons;
      dbOK = [mbOK];
      dbOKCancel = mbOKCancel;
      dbYesNo = [mbYes, mbNo];
      dbYesNoCancel = mbYesNoCancel;
      dbRetryCancel = [mbRetry, mbCancel];
      dbAbortRetryIgnore = mbAbortRetryIgnore;
    TDialogResult = type Word;
      drOK = mrOK;
      drYes = mrYes;
      drNo = mrNo;
      drCancel = mrCancel;
      drAbort = mrAbort;
      drRetry = mrRetry;
      drIgnore = mrIgnore;
  {$ENDIF}
// Fin des types de ShowDialog

  {$IFDEF SJRD_WINDOWS}
  TSoundType = (stFileName, stResource, stSysSound);
  {$ENDIF}

  TGetBooleanFunc     = function : boolean;
  TGetStringFunc      = function : string;
  TGetShortStringFunc = function : ShortString;
  TGetIntegerFunc     = function : integer;

  TSetOfChars = set of Char;
  TSetOfBytes = set of Byte;

function Dir : string;
// Renvoie le dossier dans lequel se trouve le programme
// (avec le \ ou / final)

// Fonctions de If immdiat
function IIF(Cond : boolean; Int1, Int2 : integer) : integer; overload;
function IIF(Cond : boolean; Flo1, Flo2 : Double ) : Double ; overload;
function IIF(Cond : boolean; Str1, Str2 : string ) : string ; overload;
function IIF(Cond : boolean; Obj1, Obj2 : TObject) : TObject; overload;
function IIF(Cond : boolean; Ptr1, Ptr2 : Pointer) : Pointer; overload;
function IIF(Cond : boolean; Var1, Var2 : Variant) : Variant; overload;

function Point3DToString(Point3D : T3DPoint; Delim : string = ' ') : string;
// Renvoie la chane X+Delim+Y+Delim+Z

function MinMax(Value, Min, Max : integer) : integer;
// Renvoie la valeur la plus proche de Value dans l'intervalle [Min;Max]

{$IFDEF SJRD_WINDOWS}
function ShowMes(Title, Text : string; Flags : LongWord) : integer; platform;
// Affiche un dialogue Windows via Windows.MessageBox
{$ENDIF}
function ShowDialog(Title, Text : string; DlgType : TDialogType = dtInformation;
                    DlgButtons : TDialogButtons = dbOK; DefButton : Byte = 1;
                    AddFlags : LongWord = 0) : TDialogResult;
// Dans le cas de Windows, amliore ShowMes pour accepter des paramtres comme
// ceux de MessageDlg
// Dans le cas de Linux, identique  MessageDlg mais avec moins de possibilits,
// plus lent, etc.

// L'avantage d'utiliser ShowMes et ShowDialog est que les botes de dialogue
// sont celles de Windows et mettent donc le son correspondant. L'inconvnient
// est que toutes les combinaisons de boutons ne sont pas acceptes, seules
// celles de Windows.MessageBox le sont (voir l'aide en ligne pour savoir
// quelles sont-elles).
// Si votre projet ne doit tourner que sous Linux ou que le fait d'utiliser les
// botes de dialogue Windows vous importe peu, vous aurez tout intrt 
// utiliser MessageDlg.

{$IFDEF SJRD_WINDOWS}
procedure CreateShellLink(Source, Dest : string;
                          Description : string = '';
                          IconLocation : string = '';
                          IconIndex : integer = 0;
                          Arguments : string = '';
                          WorkDir : string = '';
                          ShowCommand : integer = SW_SHOW); platform;
// Cre un raccourci avec les informations indiques
{$ENDIF}

procedure Wait(Milliseconds : integer);
// Boucle pendant Miliseconds milisecondes
procedure WaitProcessMessages(Milliseconds : integer);
// Boucle pendant Miliseconds milisecondes tout en effectuant des Application.ProcessMessages

{$IFDEF SJRD_WINDOWS}
function ExecuteSound(Sound : string; SoundType : TSoundType = stFileName;
                      Synchronous : boolean = False; Module : HMODULE = 0;
                      AddFlags : LongWord = 0) : boolean; platform;
// Excute un son via MMSytem.PlaySound
{$ENDIF}

function IntToStr0(Value, Digits : integer) : string;
// Semblable  IntToStr mais place des 0 devant pour atteindre NbreChiffres chiffres
// Attention ! IntToStr0(1000, 3) = '1000'

function ReadStrFromStream(Stream : TStream) : string;
// Lit une chane de caractres AZT dans Stream  la position courante

procedure WriteStrToStream(Stream : TStream; Str : string);
// Ecrit une chane de caractres AZT dans Stream  la position courante

function CorrectFileName(FileName : string; AcceptPathDelim : boolean = False;
                         AcceptDriveDelim : boolean = False) : boolean;
// Vrifie si FileName est un nom de fichier correct
// AcceptPathDelim indique si les \ (Windows) ou / (Linux) sont autoriss
// AcceptDriveDelim indique si les : sont accepts (Windows seulement)

function Point3D(X, Y, Z : integer) : T3DPoint;
// Renvoie un T3DPoint avec les coordonnes X, Y et Z

{$IFDEF SJRD_WINDOWS}
procedure RunURL(URL : string; Verb : string = 'open');
// Excute URL avec pour verbe Verb
// Une URL peut tre
// -> un nom de fichier : le verbe Verb est excut pour ce fichier (un fichier .txt sera ouvert dans le bloc-notes)
// -> un nom de programme : le programme est excut
// -> un adresse Internet : votre navigateur Web s'ouvre sur cette adresse
// -> 'mailto:' + une adresse E-mail : votre programme de messagerie est ouvert  l'adresse indique
// -> ...
{$ENDIF}

{$IFDEF SJRD_WINDOWS}
var ShellFolders : record
  StartMenu : string;    // Menu dmarrer
  ProgramsMenu : string; // Menu dmarrer\Programmes
  Dekstop : string;      // Bureau
  MyDocuments : string;  // Mes documents
end platform;
{$ENDIF}

procedure Register;

implementation

uses
  DateUtils;

procedure Register;
begin
end;

////////////////////////////////////////
/// Procdures et fonctions globales ///
////////////////////////////////////////

function Dir : string;
begin
  Result := IncludeTrailingPathDelimiter(ExtractFilePath(ParamStr(0)));
end;

function IIF(Cond : boolean; Int1, Int2 : integer) : integer;
begin
  if Cond then Result := Int1 else Result := Int2;
end;

function IIF(Cond : boolean; Flo1, Flo2 : Double) : Double;
begin
  if Cond then Result := Flo1 else Result := Flo2;
end;

function IIF(Cond : boolean; Str1, Str2 : string) : string;
begin
  if Cond then Result := Str1 else Result := Str2;
end;

function IIF(Cond : boolean; Obj1, Obj2 : TObject) : TObject;
begin
  if Cond then Result := Obj1 else Result := Obj2;
end;

function IIF(Cond : boolean; Ptr1, Ptr2 : Pointer) : Pointer;
begin
  if Cond then Result := Ptr1 else Result := Ptr2;
end;

function IIF(Cond : boolean; Var1, Var2 : Variant) : Variant;
begin
  if Cond then Result := Var1 else Result := Var2;
end;

function Point3DToString(Point3D : T3DPoint; Delim : string = ' ') : string;
begin
  Result := IntToStr(Point3D.X) + Delim +
            IntToStr(Point3D.Y) + Delim +
            IntToStr(Point3D.Z);
end;

function MinMax(Value, Min, Max : integer) : integer;
begin
  if Value > Max then Result := Max else
  if Value < Min then Result := Min else
  Result := Value;
end;

{$IFDEF SJRD_WINDOWS}
function ShowMes(Title, Text : string; Flags : LongWord) : integer;
var AText, ATitle : PChar;
    AFlags : LongWord;
begin
  AText  := PChar(Text);
  ATitle := PChar(Title);
  AFlags := Flags or MB_APPLMODAL; // Ajout du style Modal
  Result := MessageBox(Application.Handle, AText, ATitle, AFlags);
end;
{$ENDIF}

{$IFDEF SJRD_WINDOWS}
function ShowDialog(Title, Text : string; DlgType : TDialogType = dtInformation;
                    DlgButtons : TDialogButtons = dbOK; DefButton : Byte = 1;
                    AddFlags : LongWord = 0) : TDialogResult;
var TypeFlags, BtnsFlags, DefBtnFlags : LongWord;
begin
  case DlgType of // Transformation du paramtre DlgType en flag de type
    dtCustom       : TypeFlags := 0;
    dtInformation  : TypeFlags := MB_ICONINFORMATION;
    dtWarning      : TypeFlags := MB_ICONEXCLAMATION;
    dtError        : TypeFlags := MB_ICONERROR;
    dtConfirmation : TypeFlags := MB_ICONQUESTION;
    else TypeFlags := 0;
  end;
  case DlgButtons of // Transformation du paramtre DlgButtons en flag de boutons
    dbOK               : BtnsFlags := MB_OK;
    dbOKCancel         : BtnsFlags := MB_OKCANCEL;
    dbYesNo            : BtnsFlags := MB_YESNO;
    dbYesNoCancel      : BtnsFlags := MB_YESNOCANCEL;
    dbRetryCancel      : BtnsFlags := MB_RETRYCANCEL;
    dbAbortRetryIgnore : BtnsFlags := MB_AbortRetryIgnore;
    else BtnsFlags := MB_OK;
  end;
  case DefButton of // Transformation du paramtre DefButton en flag de bouton par dfaut
    1 : DefBtnFlags := MB_DEFBUTTON1;
    2 : DefBtnFlags := MB_DEFBUTTON2;
    3 : DefBtnFlags := MB_DEFBUTTON3;
    else DefBtnFlags := 0;
  end;
  // Appel de ShowMes et transformation du retour Word en TDialogResult
  case ShowMes(Title, Text, TypeFlags or BtnsFlags or DefBtnFlags or AddFlags) of
    IDOK     : Result := drOK    ;
    IDYES    : Result := drYes   ;
    IDNO     : Result := drNo    ;
    IDCANCEL : Result := drCancel;
    IDABORT  : Result := drAbort ;
    IDRETRY  : Result := drRetry ;
    IDIGNORE : Result := drIgnore;
    else Result := drCancel;
  end;
end;
{$ENDIF}

{$IFDEF SJRD_LINUX}
function ShowDialog(Title, Text : string; DlgType : TDialogType = dtInformation;
                    DlgButtons : TDialogButtons = dbOK; DefButton : Byte = 1;
                    AddFlags : LongWord = 0) : TDialogResult;
var NbBtns : integer;
    MsgDefButton : TMsgDlgBtn;
begin
  case DlgButtons of // Calcul du nombre de boutons
    dbOK               : NbBtns := 1;
    dbOKCancel         : NbBtns := 2;
    dbYesNo            : NbBtns := 2;
    dbYesNoCancel      : NbBtns := 3;
    dbRetryCancel      : NbBtns := 2;
    dbAbortRetryIgnore : NbBtns := 3;
    else begin NbBtns := 1 end;
  end;
  MsgDefButton := mbNone;
  if (DefButton < 1) or (DefButton > NbBtns) then DefButton := 1;
  case DefButton of // On cherche quel est le bouton par dfaut
    1 : if DlgType in [dbOK, dbOKCancel]          then MsgDefButton := mbOK     else
        if DlgType in [dbYesNo, dbYesNoCancel]    then MsgDefButton := mbYes    else
        if DlgType = dbRetryCancel                then MsgDefButton := mbRetry  else
        if DlgType = dbAbortRetryIgnore           then MsgDefButton := mbAbort;
    2 : if DlgType in [dbOKCancel, dbRetryCancel] then MsgDefButton := mbCancel else
        if DlgType in [dbYesNo, dbYesNoCancel]    then MsgDefButton := mbNo     else
        if DlgType = dbAbortRetryIgnore           then MsgDefButton := mbRetry;
    3 : if DlgType = dbYesNoCancel                then MsgDefButton := mbCancel else
        if DlgType = dbAbortRetryIgnore           then MsgDefButton := mbIgnore;
  end;
  // Appel de MessageDlg et renvoi de la valeur renvoye par celle-ci
  Result := MessageDlg(Title, Text, DlgType, DlgButtons, 0, MsgDefBtn);
end;
{$ENDIF}

{$IFDEF SJRD_WINDOWS}
procedure CreateShellLink(Source, Dest : string;
                          Description : string = '';
                          IconLocation : string = '';
                          IconIndex : integer = 0;
                          Arguments : string = '';
                          WorkDir : string = '';
                          ShowCommand : integer = SW_SHOW);
var Link : IShellLink;
begin
  // Cration de l'objet ShellLink
  Link := CreateComObject(CLSID_ShellLink) as IShellLink;
  // Fichier source
  Link.SetPath(PChar(Source));
  // Description
  if Description <> '' then
    Link.SetDescription(PChar(Description));
  // Emplacement de l'icne
  if IconLocation <> '' then
    Link.SetIconLocation(PChar(IconLocation), IconIndex);
  // Arguments
  if Arguments <> '' then
    Link.SetArguments(PChar(Arguments));
  // Dossier de travail
  if WorkDir <> '' then
    Link.SetWorkingDirectory(PChar(WorkDir));
  // Type de lancement
  Link.SetShowCmd(ShowCommand);
  // Enregistrement
  (Link as IPersistFile).Save(StringToOleStr(Dest), True);
end;
{$ENDIF}

procedure Wait(Milliseconds : integer);
var BeginTime : TDateTime;
begin
  BeginTime := Now;
  while MilliSecondsBetween(Now, BeginTime) < Milliseconds do;
end;

procedure WaitProcessMessages(Milliseconds : integer);
var BeginTime : TDateTime;
begin
  BeginTime := Now;
  while MilliSecondsBetween(Now, BeginTime) < Milliseconds do
    Application.ProcessMessages;
end;

function ExecuteSound(Sound : string; SoundType : TSoundType = stFileName;
                      Synchronous : boolean = False; Module : HMODULE = 0;
                      AddFlags : LongWord = 0) : boolean;
var Flags : LongWord;
begin
  Flags := AddFlags;
  case SoundType of
    stFileName : Flags := Flags or SND_FILENAME; // Fichier son
    stResource : Flags := Flags or SND_RESOURCE; // Ressource son
    stSysSound : Flags := Flags or SND_ALIAS;    // Alias son systme
  end;
  if not Synchronous then Flags := Flags or SND_ASYNC; // Asynchne ?
  if SoundType <> stResource then Module := 0;
  // Appel de PlaySound et renvoie de la valeur renvoye par celle-ci
  Result := PlaySound(PChar(Sound), Module, Flags);
end;

function IntToStr0(Value, Digits : integer) : string;
begin
  Result := Format('%.*d', [Digits, Value]);
end;

function ReadStrFromStream(Stream : TStream) : string;
var C : Char;
begin
  Result := '';
  repeat // Lecture caractre par caractre jusqu' trouver un caractre nul
    Stream.Read(C, 1);
    if C <> #0 then Result := Result+C;
  until C = #0;
end;

procedure WriteStrToStream(Stream : TStream; Str : string);
var I : integer;
    C : Char;
begin
  for I := 1 to Length(Str) do // criture des caractres un par un
  begin
    C := Str[I];
    Stream.Write(C, 1);
  end;
  C := #0;
  Stream.Write(C, 1); // Ajout d'un caractre nul
end;

function CorrectFileName(FileName : string; AcceptPathDelim : boolean = False;
                         AcceptDriveDelim : boolean = False) : boolean;
var I : integer;
    BadChars : set of Char;
begin
  BadChars := ['\', '/', ':', '*', '?', '"', '<', '>', '|'];
  Result := False;
  if FileName = '' then exit;
  // Si le dlimiteur de chemin est accept, on l'exclut de BadChars
  if AcceptPathDelim then
    Exclude(BadChars, PathDelim);
  // Si le dlimiteur de disque est accept, on l'exclut de BadChars
  if AcceptDriveDelim then
    Exclude(BadChars, DriveDelim);
  for I := 1 to Length(FileName) do if FileName[I] in BadChars then exit;
  Result := True;
end;

function Point3D(X, Y, Z : integer) : T3DPoint;
begin
  Result.X := X;
  Result.Y := Y;
  Result.Z := Z;
end;

{$IFDEF SJRD_WINDOWS}
procedure RunURL(URL : string; Verb : string = 'open');
begin
  ShellExecute(GetDesktopWindow(), PChar(Verb), PChar(URL), nil, nil, SW_SHOWNORMAL);
end;
{$ENDIF}

/////////////////////////
/// Classe ELastError ///
/////////////////////////

constructor EAPIError.Create(Error : integer);
begin
  // Le message est rcupr via SysErrorMessage
  inherited Create(SysErrorMessage(Error));
  FErrorCode := Error;
end;

constructor EAPIError.Create;
begin
  // Le code erreur est rcupr via GetLastError
  Create(GetLastError);
end;

initialization
{$IFDEF SJRD_WINDOWS}
  with TRegistry.Create do
  begin
    RootKey := HKEY_CURRENT_USER;
    // Ouverture de la cl contenant les dossiers du Shell
    OpenKey('Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders', False);
    // Lecture de tous les chemins utiliss
    ShellFolders.StartMenu    := IncludeTrailingPathDelimiter(ReadString('Start Menu'));
    ShellFolders.ProgramsMenu := IncludeTrailingPathDelimiter(ReadString('Programs'  ));
    ShellFolders.Dekstop      := IncludeTrailingPathDelimiter(ReadString('Desktop'   ));
    ShellFolders.MyDocuments  := IncludeTrailingPathDelimiter(ReadString('Personal'  ));
    Free;
  end;
{$ENDIF}
end.
